/**
 * Copyright (C) 2008-2012 AgroSense Foundation.
 *
 * AgroSense is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * There are special exceptions to the terms and conditions of the GPLv3 as it is applied to
 * this software, see the FLOSS License Exception
 * <http://www.agrosense.eu/foss-exception.html>.
 *
 * AgroSense is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with AgroSense.  If not, see <http://www.gnu.org/licenses/>.
 */
package nl.cloudfarming.client.stock.crop.visual;

import javax.swing.JToolBar;
import nl.cloudfarming.client.crop.model.Classification;
import nl.cloudfarming.client.crop.model.CropDefinition;
import nl.cloudfarming.client.crop.model.Variety;
import nl.cloudfarming.client.stock.crop.actions.AddCropDefinitionAction;
import nl.cloudfarming.client.stock.crop.actions.RemoveAction;
import org.openide.util.Lookup;
import org.openide.util.LookupEvent;
import org.openide.util.LookupListener;

/**
 * Toolbar to store all actions possible inside the croplist. Registers itself
 * as lookuplistener of cropdefinition, classification and variety.
 *
 * @author Gerben Feenstra & Merijn Zengers
 */
class CropListToolBar extends JToolBar {

    private final Listener lookupListener = new Listener();
    private Lookup.Result<CropDefinition> cropDefLookupResult;
    private Lookup.Result<Classification> classificationLookupResult;
    private Lookup.Result<Variety> varietyLookupResult;
    private RemoveAction removeAction;
    private AddCropDefinitionAction addCropAction;
    private Lookup lookup;

    /**
     * Constructs toolbar, adds actions to it
     */
    public CropListToolBar(Lookup lookup) {
        this.lookup = lookup;
        initActions();
        initLookupResults();
    }

    private void initActions() {
        initRemoveAction();
        initAddCropAction();
    }

    private void initRemoveAction() {
        removeAction = new RemoveAction(lookup);
        removeAction.setEnabled(false);
        add(removeAction);
    }

    private void initAddCropAction() {
        addCropAction = new AddCropDefinitionAction();
        addCropAction.setEnabled(true);
        add(addCropAction);
    }

    private void initLookupResults() {
        initLookupResult(cropDefLookupResult, CropDefinition.class);
        initLookupResult(classificationLookupResult, Classification.class);
        initLookupResult(varietyLookupResult, Variety.class);
    }

    private void initLookupResult(Lookup.Result result, Class clazz) {
        result = lookup.lookupResult(clazz);
        result.addLookupListener(lookupListener);
    }

    /**
     * Listens to lookup changes if CropDefinition, classification or variety
     * are in the lookup it will enable the suitable actions
     */
    private class Listener implements LookupListener {

        @Override
        public void resultChanged(LookupEvent ev) {
            removeAction.setEnabled(false);

            Lookup.Result<Object> r = (Lookup.Result<Object>) ev.getSource();
            for (Object o : r.allInstances()) {
                if (o instanceof CropDefinition || o instanceof Classification || o instanceof Variety) {
                    removeAction.setEnabled(true);
                    break;
                }
            }
        }
    }
}