/**
 * Copyright (C) 2008-2012 AgroSense Foundation.
 *
 * AgroSense is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * There are special exceptions to the terms and conditions of the GPLv3 as it is applied to
 * this software, see the FLOSS License Exception
 * <http://www.agrosense.eu/foss-exception.html>.
 *
 * AgroSense is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with AgroSense.  If not, see <http://www.gnu.org/licenses/>.
 */
package nl.cloudfarming.client.stock.crop.visual;

import java.awt.BorderLayout;
import java.awt.Color;
import javax.swing.ActionMap;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JToolBar;
import nl.cloudfarming.client.crop.model.Crop;
import nl.cloudfarming.client.crop.model.CropDefinition;
import nl.cloudfarming.client.model.AgroURI;
import nl.cloudfarming.client.model.EnterpriseIdProvider;
import org.openide.explorer.ExplorerManager;
import org.openide.explorer.ExplorerUtils;
import org.openide.explorer.view.TreeTableView;
import org.openide.nodes.Node;
import org.openide.nodes.PropertySupport;
import org.openide.util.Exceptions;
import org.openide.util.Lookup;
import org.openide.util.NbBundle;

/**
 * Visual component to display a treetableview of the croplist with a toolbar on
 * top. Uses explorermanager to get the rootnode.
 *
 * @author Timon Veenstra <monezz@gmail.com>
 * @author Wytse Visser
 */
@NbBundle.Messages({
    "displayName_PROP_NAME=Name",
    "displayName_PROP_SOURCE=Source",
    "displayName_PROP_COLOR=Color"
})
public class CropListVisualPanel extends JPanel implements ExplorerManager.Provider, Lookup.Provider {

    private final ExplorerManager explorerManager;
    private final TreeTableView treeView = new TreeTableView();
    private Lookup lookup;

    public CropListVisualPanel(ExplorerManager explorerManager) {
        this.explorerManager = explorerManager;
        setLayout(new BorderLayout());

        lookup = createLookup();
        add(createScrollPane());
        initData();
    }

    private Lookup createLookup() {
        return ExplorerUtils.createLookup(explorerManager, new ActionMap());
    }

    private JScrollPane createScrollPane() {
        JScrollPane scrollPane = new JScrollPane();
        scrollPane.setColumnHeaderView(createToolbar());
        scrollPane.setViewportView(treeView);

        return scrollPane;
    }

    private JToolBar createToolbar() {
        JToolBar toolbar = new CropListToolBar(lookup);
        toolbar.setFloatable(false);
        return toolbar;
    }

    private void initData() {
        try {
            treeView.setProperties(createDummyProperties());
            treeView.setRootVisible(false);
        } catch (NoSuchMethodException ex) {
            Exceptions.printStackTrace(ex);
        }
    }

    private static Node.Property[] createDummyProperties() throws NoSuchMethodException {
        final EnterpriseIdProvider EIP = new AgroURI("AGRONL/000000000/CTN/1");
        CropDefinition dummy = new CropDefinition(EIP, new Crop(EIP, "DUMMY_CROP"));

        Node.Property nameProp = new PropertySupport.Reflection(dummy, String.class, CropDefinition.PROP_NAME);
        nameProp.setDisplayName(Bundle.displayName_PROP_NAME());
        nameProp.setValue("ComparableColumnTTV", Boolean.TRUE); //NOI18N
        nameProp.setValue("SortingColumnTTV", Boolean.TRUE); //NOI18N

        Node.Property sourceProp = new PropertySupport.Reflection(dummy, String.class, "getSource", null);
        sourceProp.setDisplayName(Bundle.displayName_PROP_SOURCE());

        Node.Property colorProp = new PropertySupport.Reflection(dummy, Color.class, "getColor", null);
        colorProp.setDisplayName(Bundle.displayName_PROP_COLOR());

        return new Node.Property[]{nameProp, sourceProp, colorProp};
    }

    @Override
    public ExplorerManager getExplorerManager() {
        return explorerManager;
    }

    boolean valid() {
        return true;
    }

    @Override
    public Lookup getLookup() {
        return lookup;
    }
}
