/**
 * Copyright (C) 2008-2012 AgroSense Foundation.
 *
 * AgroSense is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * There are special exceptions to the terms and conditions of the GPLv3 as it is applied to
 * this software, see the FLOSS License Exception
 * <http://www.agrosense.eu/foss-exception.html>.
 *
 * AgroSense is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with AgroSense.  If not, see <http://www.gnu.org/licenses/>.
 */
package nl.cloudfarming.client.stock.crop.visual;

import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.util.Set;
import javax.swing.JComponent;
import nl.cloudfarming.client.crop.model.Crop;
import nl.cloudfarming.client.crop.model.CropBean;
import nl.cloudfarming.client.crop.model.CropDefinition;
import nl.cloudfarming.client.crop.model.CropDefinitionManager;
import nl.cloudfarming.client.crop.model.CropManager;
import nl.cloudfarming.client.stock.crop.storage.PropertyChangeTracker;
import nl.cloudfarming.client.stock.crop.visual.nodes.CropsChildFactory;
import nl.cloudfarming.client.stock.crop.visual.nodes.CropsRootNode;
import org.netbeans.spi.options.OptionsPanelController;
import org.openide.explorer.ExplorerManager;
import org.openide.util.HelpCtx;
import org.openide.util.Lookup;

@OptionsPanelController.TopLevelRegistration(
    categoryName = "#OptionsCategory_Name_CropList",
iconBase = "nl/cloudfarming/client/stock/crop/box.png",
keywords = "#OptionsCategory_Keywords_CropList",
keywordsCategory = "Crop List")
@org.openide.util.NbBundle.Messages({"OptionsCategory_Name_CropList=Crop list", "OptionsCategory_Keywords_CropList=Crop list"})
public final class CroplistOptionsPanelController extends OptionsPanelController {

    private CropListVisualPanel panel;
    private final PropertyChangeSupport pcs = new PropertyChangeSupport(this);
    private boolean changed;
    private PropertyChangeTracker propertyChangeTracker = new PropertyChangeTracker();

    @Override
    public void update() {
        changed = false;
    }

    @Override
    public void applyChanges() {
        Set<CropBean> changedCropBeans = propertyChangeTracker.getUpdatedCropBeans();
        for (CropBean cropBean : changedCropBeans) {
            if (cropBean instanceof CropDefinition) {
                Lookup.getDefault().lookup(CropDefinitionManager.class).save((CropDefinition) cropBean);
            }
            if (cropBean instanceof Crop) {
                Lookup.getDefault().lookup(CropManager.class).save((Crop) cropBean);
            }
        }
        propertyChangeTracker.reset();
        changed = false;
    }

    @Override
    public void cancel() {
        propertyChangeTracker.undoPropertyChanges();
    }

    @Override
    public boolean isValid() {
        return getPanel().valid();
    }

    @Override
    public boolean isChanged() {
        return changed;
    }

    @Override
    public HelpCtx getHelpCtx() {
        return new HelpCtx("nl.cloudfarming.client.stock.crop.croplist");
    }

    @Override
    public JComponent getComponent(Lookup masterLookup) {
        return getPanel();
    }

    @Override
    public void addPropertyChangeListener(PropertyChangeListener l) {
        pcs.addPropertyChangeListener(l);
    }

    @Override
    public void removePropertyChangeListener(PropertyChangeListener l) {
        pcs.removePropertyChangeListener(l);
    }

    private CropListVisualPanel getPanel() {
        if (panel == null) {
            panel = new CropListVisualPanel(createExplorerManager());
        }
        return panel;
    }

    private ExplorerManager createExplorerManager() {
        CropsRootNode rootNode = new CropsRootNode(new CropsChildFactory(propertyChangeTracker));
        ExplorerManager explorerManager = new ExplorerManager();
        explorerManager.setRootContext(rootNode);

        return explorerManager;
    }

    void changed() {
        if (!changed) {
            changed = true;
            pcs.firePropertyChange(OptionsPanelController.PROP_CHANGED, false, true);
        }
        pcs.firePropertyChange(OptionsPanelController.PROP_VALID, null, null);
    }
}
