/**
 * Copyright (C) 2008-2012 AgroSense Foundation.
 *
 * AgroSense is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * There are special exceptions to the terms and conditions of the GPLv3 as it is applied to
 * this software, see the FLOSS License Exception
 * <http://www.agrosense.eu/foss-exception.html>.
 *
 * AgroSense is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with AgroSense.  If not, see <http://www.gnu.org/licenses/>.
 */
package nl.cloudfarming.client.stock.crop.visual.nodes;

import java.beans.IntrospectionException;
import javax.swing.Action;
import nl.cloudfarming.client.crop.model.*;
import nl.cloudfarming.client.stock.crop.actions.RemoveAction;
import org.openide.nodes.BeanNode;
import org.openide.nodes.Children;
import org.openide.nodes.Node;
import org.openide.util.HelpCtx;
import org.openide.util.lookup.Lookups;

/**
 * Node which represents a cropdefinition.
 *
 * @author Timon Veenstra <monezz@gmail.com>
 */
public class CropDefinitionNode extends BeanNode<CropDefinition> {

    private CropDefinition cropDefinition;
    private static final String CROPDEFINITION_ICON = "nl/cloudfarming/client/stock/crop/cropdefinition.png";

    /**
     * Construct node from cropdefinition, puts cropdefinition in its lookup.
     *
     * @param cropDefinition
     * @throws IntrospectionException
     */
    public CropDefinitionNode(CropDefinition cropDefinition) throws IntrospectionException {
        super(cropDefinition, new CropDefinitionChildren(cropDefinition), Lookups.fixed(cropDefinition));
        this.cropDefinition = cropDefinition;
        this.setIconBaseWithExtension(CROPDEFINITION_ICON);
    }

    /**
     * Returns the actions that can be invoked on this cropDefinitionNode
     *
     * @param context Not used
     * @return Array of actions for this node
     */
    @Override
    public Action[] getActions(boolean context) {
        Action[] actions = new Action[1];
        actions[0] = new RemoveAction(getLookup());
        return actions;
    }

    @Override
    public Action getPreferredAction() {
        return null;
    }

    @Override
    public HelpCtx getHelpCtx() {
        return new HelpCtx("nl.cloudfarming.client.stock.crop.croplist");
    }

    /**
     * Returns the cropdefinition belonging to this node.
     *
     * @return The cropdefinition
     */
    public CropDefinition getCropDefinition() {
        return cropDefinition;
    }

    /**
     * Creates children (varieties and classifications) for a cropdefinition
     * node
     */
    private static class CropDefinitionChildren extends Children.Keys<CropDefinition> {

        public CropDefinitionChildren(CropDefinition bean) {
            setKeys(new CropDefinition[]{bean});
            addNotify();
        }

        @Override
        protected Node[] createNodes(CropDefinition cropDefinition) {
            Node classificationAggregateNode = new ClassificationAggregateNode(cropDefinition, Lookups.fixed(cropDefinition));
            Node varietyAggregateNode = new VarietyAggregateNode(cropDefinition, Lookups.fixed(cropDefinition));
            return new Node[]{classificationAggregateNode, varietyAggregateNode};
        }
    }
}
