/**
 * Copyright (C) 2008-2012 AgroSense Foundation.
 *
 * AgroSense is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * There are special exceptions to the terms and conditions of the GPLv3 as it is applied to
 * this software, see the FLOSS License Exception
 * <http://www.agrosense.eu/foss-exception.html>.
 *
 * AgroSense is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with AgroSense.  If not, see <http://www.gnu.org/licenses/>.
 */
package nl.cloudfarming.client.stock.crop.visual.nodes;

import java.beans.IntrospectionException;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.List;
import nl.cloudfarming.client.crop.model.CropDefinition;
import nl.cloudfarming.client.stock.crop.storage.CropDefinitionStorage;
import nl.cloudfarming.client.stock.crop.storage.PropertyChangeTracker;
import org.openide.nodes.ChildFactory;
import org.openide.nodes.Node;
import org.openide.util.Exceptions;

/**
 * Factory to create children for the (invisible) root Crops node. 
 * 
 * @author Gerben Feenstra
 */
public class CropsChildFactory extends ChildFactory<CropDefinition> implements PropertyChangeListener {

    private PropertyChangeTracker propertyChangeTracker;
    
    public CropsChildFactory(PropertyChangeTracker tracker) {
        this.propertyChangeTracker = tracker;
        CropDefinitionStorage.getInstance().addPropertyChangeListener(CropDefinitionStorage.PROP_CROP_DEFINITION_ADDREMOVE, this);
    }

    @Override
    protected boolean createKeys(List<CropDefinition> toPopulate) {
        toPopulate.addAll(CropDefinitionStorage.getInstance().getCropDefinitions());
        addPropertyChangeTracker(toPopulate);
        return true;
    }

    @Override
    protected Node createNodeForKey(CropDefinition key) {
        try {
            return new CropDefinitionNode(key);
        } catch (IntrospectionException ex) {
            Exceptions.printStackTrace(ex);
            return null;
        }
    }

    @Override
    public void propertyChange(PropertyChangeEvent evt) {
        this.refresh(true);
    }
    
    private void addPropertyChangeTracker(List<CropDefinition> cropDefinitions) {
        for(CropDefinition cropDefinition : cropDefinitions) {
            cropDefinition.addPropertyChangeListener(propertyChangeTracker);
        }
    }
}